/*
*  Bookmark Duplicate Detector
*
* Author: Stphane BERTIN
* http://bookmarkdd.mozdev.org
*
* Originally created by Paradox v0.0.1
* Ported to Firefox 1.5 by Stefano Caselli v0.0.5 and v0.0.7
* Since v0.0.7 Stphane BERTIN is the developer
*
* Principal changes:
* 0.2.2: Support for firefox 1.0 added
* 0.3: Add "Search Duplicate Bookmark" function
* 0.4: Rewrite to avoid conflict with others extensions, searchBar feature improved
* 0.5: add autoDelete duplicates, improve customization of menus, add Move button
*
* TODO:
*    Remove Import test function before new release !!!!!
*/
//----------------------------------------------------------------------------

//Used to include only one time bookmark.js and avoid error message about already specified constant
try{
    if(ADD_BM_DIALOG_FEATURES) {}
} catch(e){
    var loader =  Components.classes["@mozilla.org/moz/jssubscript-loader;1"]
		    .getService(Components.interfaces.mozIJSSubScriptLoader);
    loader.loadSubScript("chrome://browser/content/bookmarks/bookmarks.js");
}

//Init Bookmarks.js Service
initServices();
initBMService();

//gNC_NS is defined since Firefox 1.5 --> NC_NS === undefined
//NC_NS is defined for Firefox 1.0 --> gNC_NS === undefined

const gNCNS = "http://home.netscape.com/NC-rdf#";

var Bookmarkdd = {

    //Default version is Firefox 1.5 :-)
    version: "1.5",
    // Used to DEBUG
    DEBUG: true,
    userResponse: false,

    // Used by drag&drop
    insertAndCheckURL: function (aAction, aSelection, aTarget, aTargetIndex)
    {
	var DuplicatesBundle = document.getElementById('duplicates-strings');
	if (!DuplicatesBundle) {
	    // Used by BookmarksPanelOverlay
	    DuplicatesBundle = top.document.getElementById('duplicates-strings');
	}
	var isValid = BookmarksUtils.isSelectionValidForInsertion(aSelection, aTarget);
	if (!isValid) {
	    SOUND.beep();
	    return false;
	}
	//XXX: paste seems to not work ! It is not possible to find URL with paste !
	if (aAction == "drag" || aAction == "paste"){
	    var itemValue = BMDS.GetTarget(aSelection.item[0], RDF.GetResource(gNCNS+"URL"), true);
	    try {
		var url = itemValue.QueryInterface(Components.interfaces.nsIRDFLiteral).Value;
		if (Bookmarkdd.DEBUG) Bookmarkdd.myDump("insertAndCheckURL + aAction: "+aAction+"\nurl = "+url);

		var isDuplicate = Bookmarkdd.isBookmarked(url);
		if(isDuplicate) {
		    var message = DuplicatesBundle.getString("alreadyIn-Folder")+"\n";
		    message += isDuplicate.parents.join("\n")+"\n";
		    message += DuplicatesBundle.getString("alreadyIn-Confirm");
		    if(! BddConfirm(message, isDuplicate.RDFnodes)) return false;
		}
	    } catch(e){
		if (Bookmarkdd.DEBUG) Bookmarkdd.myDump("insertAndCheckURL + aAction: "+aAction+"\nNo url found !");
	    }
	} else if (Bookmarkdd.DEBUG) {
		Bookmarkdd.myDump("DEBUG: Bookmarkdd.insertAndCheckURL + aAction: "+aAction);
	}

	BookmarksUtils.insertSelection(aAction, aSelection, aTarget, aTargetIndex);
	BookmarksUtils.flushDataSource();
	if (Bookmarkdd.version == "1.5") BookmarksUtils.refreshSearch();

	return true;
    },


    // Verify if the specified URL is already in Bookmark
    // If URL already in Bookmark function return an Array of Parent folder
    isBookmarked: function (aUrl,aUrlFeed)
    {
	var DuplicatesBundle = document.getElementById('duplicates-strings');

	if ( ! BMSVC || ! BMDS || ! RDF || ! gNCNS ) {
	    alert(DuplicatesBundle.getString("flagrantError-Alert"));
	    return false; 
	}

	var urlArcID = RDF.GetResource(gNCNS+"ID");
	var FeedArc = RDF.GetResource(gNCNS+"FeedURL");
	if (!aUrlFeed){
	    var urlLiteral = RDF.GetLiteral(aUrl);
	    var bmResource = BMSVC.GetSources(RDF.GetResource(gNCNS+"URL"), urlLiteral, true);
	} else {
	    var urlLiteral = RDF.GetLiteral(aUrlFeed);
	    var bmResource = BMSVC.GetSources(RDF.GetResource(gNCNS+"FeedURL"), urlLiteral, true);
	}

	var count = 0;
	var URLFound = new URLConstructor();

	while (bmResource.hasMoreElements()) {
	    var DuplicateURLresource=bmResource.getNext();
	    var URLParent = BMSVC.getParent(DuplicateURLresource);
	    if(URLParent) {
		if (!aUrlFeed){
		//urlRes can't have a FeedURL if aUrlFeed is not defined !!
		    var targetFeed = BMDS.GetTarget(DuplicateURLresource, FeedArc, true);
		    if (targetFeed) continue;
		}

		URLFound.parents[count] = this.findPath(DuplicateURLresource);
		//Store RDF node ID for Bookmark manipulation: URLFound.RDFnodes[count].Value =  rdf:#$...
		URLFound.RDFnodes[count] = BMDS.GetTarget(DuplicateURLresource.QueryInterface(kRDFRSCIID), urlArcID, true);
		if(!URLFound.RDFnodes[count]) {
		    // If null it means that the Resource is not already include within RDF document
		    // and DuplicateURLresource is a RDF Node resource.
		    URLFound.RDFnodes[count] = DuplicateURLresource; //DuplicateURLresource.Value = rdf:#$...
		}
		count++;
	    }
	}

	if (count>0) return URLFound;
	return false;
    },

    // Replace addBookmark to verify if URL is duplicated
    addBookmarkURL: function (aURL, aTitle, aCharset, aIsWebPanel, aDescription)
    {
	var DuplicatesBundle = document.getElementById('duplicates-strings');
	var dArgs = {
		name: aTitle,
		url: aURL,
		charset: aCharset,
		bWebPanel: aIsWebPanel,
		description: aDescription
	}

	var isDuplicate = Bookmarkdd.isBookmarked(aURL);
	if(isDuplicate) {
	    var message = DuplicatesBundle.getString("alreadyIn-Folder")+"\n";
	    message += isDuplicate.parents.join("\n")+"\n";
	    message += DuplicatesBundle.getString("alreadyIn-Confirm");
	    if(! BddConfirm(message, isDuplicate.RDFnodes)) return false;
	}

	if(Bookmarkdd.version == "1.5") {
	    return window.openDialog("chrome://browser/content/bookmarks/addBookmark2.xul", "", ADD_BM_DIALOG_FEATURES, dArgs);
	} else if (Bookmarkdd.version == "1.0") {
	    return window.openDialog("chrome://browser/content/bookmarks/addBookmark2.xul", "", "centerscreen,chrome,dialog,resizable,dependent", aTitle,aURL,null,aCharset,null,null,aIsWebPanel);
	}

    },

    addLivemark: function (aURL, aFeedURL, aTitle, aDescription)
    {
	var DuplicatesBundle = document.getElementById('duplicates-strings');
	var dArgs = {
	    name: aTitle,
	    url: aURL,
	    bWebPanel: false,
	    feedURL: aFeedURL,
	    description: aDescription
	};
	try {
	    var toolbarFolderURI = BMDS.getBookmarksToolbarFolder().Value;
	    dArgs.folderURI = toolbarFolderURI;
	} catch (e) { 
	}
	//window.openDialog("chrome://browser/content/bookmarks/addBookmark2.xul", "", ADD_BM_DIALOG_FEATURES, dArgs);

	var isDuplicate = Bookmarkdd.isBookmarked(aURL, aFeedURL);
	if(isDuplicate) {
	    var message = DuplicatesBundle.getString("alreadyIn-Folder")+"\n";
	    message += isDuplicate.parents.join("\n")+"\n";
	    message += DuplicatesBundle.getString("alreadyIn-Confirm");
	    if(! BddConfirm(message, isDuplicate.RDFnodes)) return false;
	}

	if(Bookmarkdd.version == "1.5") {
	    return window.openDialog("chrome://browser/content/bookmarks/addBookmark2.xul", "", ADD_BM_DIALOG_FEATURES, dArgs);
	} else if (Bookmarkdd.version == "1.0") {
	    return window.openDialog("chrome://browser/content/bookmarks/addBookmark2.xul", "", "centerscreen,chrome,dialog,resizable,dependent", aTitle,aURL,null,null,null,null,false,null,null,null,aFeedURL);
	}
    },

    // Function used to search in ALL Bookmarks which URL are duplicated
    searchDuplicateURL: function ()
    {
	var DuplicatesBundle = document.getElementById('duplicates-strings');
	var urlArc = RDF.GetResource(gNCNS+"URL");
	var FeedArc = RDF.GetResource(gNCNS+"FeedURL");

	var AllBookmarksResources = BMDS.GetAllResources();

	var AllURLs = new Array(), URLDuplicates = new Array();
	var AllURLsTotal = 0, URLdupNum = 0;

	//Bookmark{URL}
	//Livemark{URL, FeedURL} So livemark is also detected by Bookmark target.
	while (AllBookmarksResources.hasMoreElements()) {
	    var urlRes = AllBookmarksResources.getNext().QueryInterface(kRDFRSCIID);

	    //if Parent's Folder is a Livemark don't verify URL go to next Resource
	    var aParent = BMSVC.getParent(urlRes);
	    if (aParent) var ptype = BookmarksUtils.resolveType(aParent);
	    if (ptype == "Livemark") continue;

	    var target = BMDS.GetTarget(urlRes, urlArc, true);
	    //Is urlRes a RSS folder ?
	    var targetFeed = BMDS.GetTarget(urlRes, FeedArc, true);
	    if (targetFeed) {
//var targetURL = target.QueryInterface(kRDFLITIID).Value;
	    	var targetFeedURL = targetFeed.QueryInterface(kRDFLITIID).Value;
		//URLFeed still detected even if it has been removed during same Firefox session
		// so we need to verify if it is really a duplicate Resource
//var FeedDuplicate = Bookmarkdd.isBookmarked(targetURL,targetFeedURL);
		var FeedDuplicate = Bookmarkdd.isBookmarked("",targetFeedURL);
		if (FeedDuplicate && FeedDuplicate.parents.length < 2)  continue;
		target = targetFeed;
	    }

	    //var urlArcID   = RDF.GetResource(gNCNS+"ID");
	    //var Node = BMDS.GetTarget(urlRes, urlArcID, true);
	    //if (target && ptype != "Livemark"){
	    if (target){
		var url = target.QueryInterface(kRDFLITIID).Value;
		if(Bookmarkdd.version == "1.5"){
		    if (AllURLs.indexOf(url) == -1){
			    AllURLs[AllURLsTotal] = url;
		    } else {
			    if (URLDuplicates.indexOf(url) == -1) {
				    URLDuplicates[URLdupNum] = url;
				    URLdupNum++;
			    }
		    }
		} else { //Bookmarkdd.version == "1.0"
		    AllURLs[AllURLsTotal] = url;
		}
		AllURLsTotal++;
	    }

	}
	if  (Bookmarkdd.version == "1.0"){
	    var AllURLnum = AllURLs.length, isDuplicate = false;
	    var rssFeed = /(.rss)$/i;	//Verify if url is a FeedURL
	    for (var countB = 0; countB < AllURLnum;countB++){
		if (!AllURLs[countB]) continue;

		url = AllURLs[countB];
		var urlFeed = rssFeed.exec(url);

		if (!urlFeed) {
		    isDuplicate = Bookmarkdd.isBookmarked(url);
		} else {
		    isDuplicate = Bookmarkdd.isBookmarked(url,url);
		}
		if (isDuplicate && isDuplicate.parents.length > 1){
		    var alreadyIN = false;
		    for (var count=0, URLnum=URLDuplicates.length; count<URLnum; count++){
			if (URLDuplicates[count] == url) alreadyIN = true;
		    }
		    if (!alreadyIN){
			URLDuplicates[URLdupNum] = url;
			URLdupNum++;
		    } else { var removed = AllURLs.splice(countB, 1); } //Remove each duplicate to decrease loop round
		}
	    }
	}

	if (URLdupNum >0 ){
	    URLDuplicates.sort();
	    window.openDialog("chrome://duplicates/content/ManageDuplicates.xul", "bookmarkDD-ManageDuplicates", "chrome,alwaysRaised,resizable,centerscreen,dialog=no", URLDuplicates,URLdupNum,AllURLsTotal);
	} else { alert(DuplicatesBundle.getString("SearchDup-none")); }
    },


    // Return a chain of URL Parent Folder Name
    // getParentChain doesn't work !!
    findPath: function (URLresource)
    {
	var ParentResource = URLresource;
	var PathArray = new Array();
	var boucle = true, ParentName;
/*XXX Maybe the following is faster
	//Extract from bookmarksTree.xml
	var chain = BMSVC.getParentChain(aResource);
	Bookmarkdd.myDump("Chain length= "+chain.length+"\n");
	for (var i=0; i<chain.length; i++) {
	    var rParent = chain.queryElementAt(i, kRDFRSCIID);
	    Bookmarkdd.myDump(i+":"+BookmarksUtils.getProperty(rParent, gNC_NS+"Name"));
	}
*/
	while (boucle){
	    ParentResource = BMSVC.getParent(ParentResource);
	    if (ParentResource) {
		ParentName = BookmarksUtils.getProperty(ParentResource.Value,gNCNS+"Name");
		if (ParentName) PathArray.push(ParentName);
	    } else {
		boucle = false;
	    }
	}
	PathArray.reverse();

	return PathArray.join(" > ");
    },


    //This function is executed when Option panel unload
    VerifyMenuItemStatus: function ()
    {
	var wm = Components.classes["@mozilla.org/appshell/window-mediator;1"]
		    .getService(Components.interfaces.nsIWindowMediator);
	var enumerator = wm.getEnumerator("");
	var enumBreak = 0;

	while(enumerator.hasMoreElements()) {
	    var win = enumerator.getNext();
	    // |win| is [Object ChromeWindow] (just like |window|), do something with it
	    // Verify if BookmarksMenu's hidden property is synchronised with UserPrefs
	    var BookmarksMenu=win.document.getElementById("BookmarkDD-context-menu");
	    if(BookmarksMenu) {
		if(!UserPrefs.getPref("bool","BookmarksMenu.addLink")) {
		    BookmarksMenu.setAttribute('hidden',true);
		} else {
		    BookmarksMenu.setAttribute('hidden',false);
		}
		enumBreak++;
	    }
	    // Verify if ToolsMenu's hidden property is synchronised with UserPrefs
	    var ToolsMenu=win.document.getElementById("BookmarkDD-PanelLink");
	    if(ToolsMenu) {
		if(!UserPrefs.getPref("bool","ToolsMenu.addLink")) {
		    ToolsMenu.setAttribute('hidden',true);
		} else {
		    ToolsMenu.setAttribute('hidden',false);
		}
		enumBreak++;
	    }
	    if(enumBreak>0) break;
	}

    },

    HideMenuItem: function ()
    {
	try{
	    var menu=document.getElementById("bookmarks-menu");
	    if(!menu) return; //We are not in windowsID="main-window"
	    if(!UserPrefs.getPref("bool","BookmarksMenu.addLink")) document.getElementById("BookmarkDD-context-menu").setAttribute('hidden',true);
	    if(!UserPrefs.getPref("bool","ToolsMenu.addLink")) document.getElementById("BookmarkDD-PanelLink").setAttribute('hidden',true);
	}catch(e){alert(e); return;}

    },


    // With a RDF node generate a Selection used by BookmarksCommand
    createSelection: function (aNode)
    {
	var items = new Array();
	items[0] = aNode;//URLrdfNodes[aItem.value];
	var selection = {item: items, parent:Array(1), length: 1, prop:Array(0)};
	selection.parent[0] = BMDS.getParent(selection.item[0]);
	selection.isContainer = new Array(selection.length); //It is not a container !!

	return selection;
    },

    //Function used to log message within JS Console
    myDump: function (aMessage) 
    {
	var consoleService = Components.classes["@mozilla.org/consoleservice;1"]
				 .getService(Components.interfaces.nsIConsoleService);
	consoleService.logStringMessage("BookmarkDD's message: " + aMessage);
    },

    // Function used to find localized Strings due to restriction of overlay: element in overlay have no ID !!
    // Insertion of <stringbundle id="duplicates-strings"/> can't be called !!!!!
    searchStringLocalisation: function (aLabel){
	var wm = Components.classes["@mozilla.org/appshell/window-mediator;1"]
		    .getService(Components.interfaces.nsIWindowMediator);
	var enumerator = wm.getEnumerator("");

	while(enumerator.hasMoreElements()) {
	    var win = enumerator.getNext();
	    // |win| is [Object ChromeWindow] (just like |window|), do something with it
	    var duplicatesStrings=win.document.getElementById("duplicates-strings");
	    if(duplicatesStrings) {
		return duplicatesStrings.getString(aLabel);
	    }
	}
	return false;
    },

/*****************************************************************************
    TODO: FOLLOWING LINES ARE USED ONLY FOR TEST with import Bookmark's file
    Remove them before release new version !!!!!!!!!!!!
    Until comment: "END OF IMPORT TEST"
*****************************************************************************/
//Import Bookmark
// Execute a command with the given source and arguments
  doBookmarksCommand: function (aSource, aCommand, aArgumentsArray, dataSource)
  {
    var rCommand = RDF.GetResource(aCommand);
  
    var kSuppArrayContractID = "@mozilla.org/supports-array;1";
    var kSuppArrayIID = Components.interfaces.nsISupportsArray;
    var sourcesArray = Components.classes[kSuppArrayContractID].createInstance(kSuppArrayIID);
    if (aSource) {
      sourcesArray.AppendElement(aSource);
    }
  
    var argsArray = Components.classes[kSuppArrayContractID].createInstance(kSuppArrayIID);
    var length = aArgumentsArray?aArgumentsArray.length:0;
    for (var i = 0; i < length; ++i) {
      var rArc = RDF.GetResource(aArgumentsArray[i].property);
      argsArray.AppendElement(rArc);
      var rValue = null;
      if ("resource" in aArgumentsArray[i]) { 
        rValue = RDF.GetResource(aArgumentsArray[i].resource);
      }
      else
        rValue = RDF.GetLiteral(aArgumentsArray[i].literal);
      argsArray.AppendElement(rValue);
    }

    // Exec the command in the Bookmarks datasource. 
    var kBMSVCIID = Components.interfaces.nsIBookmarksService;
    inSVC = dataSource.QueryInterface(kBMSVCIID);
    var result_message = inSVC.importBookmarks(argsArray);
    //var result_message = dataSource.DoCommand(sourcesArray, rCommand, argsArray);
    if (result_message) alert("DoCommand's message: "+result_message);
  },

  importBookmarksFromFile: function ()
  {
    /*var loader =  Components.classes["@mozilla.org/moz/jssubscript-loader;1"]
		    .getService(Components.interfaces.mozIJSSubScriptLoader);
    loader.loadSubScript("chrome://duplicates/content/rdfds.js");
    var rdfService = Components.classes["@mozilla.org/rdf/rdf-service;1"].
                   getService(Components.interfaces.nsIRDFService);
    */

    //var inmemds = Components.classes["@mozilla.org/rdf/datasource;1?name=in-memory-datasource"]
                //.createInstance(Components.interfaces.nsIRDFDataSource);
    //var kBMSVCIID = Components.interfaces.nsIBookmarksService;
    //var dsBMSVC = inmemds.QueryInterface(kBMSVCIID);
    //inmemds.Assert(rTargetDup, RDF.GetResource("http://home.netscape.com/NC-rdf#child"), RDF.GetLiteral("urn:1"), true);

    ///transaction...
open("chrome://duplicates/content/ImportBookmarks.xul", "BookmarkDD-Import", "chrome,alwaysRaised,resizable,centerscreen", fileName);
return;
    try {
      const kFilePickerContractID = "@mozilla.org/filepicker;1";
      const kFilePickerIID = Components.interfaces.nsIFilePicker;
      const kFilePicker = Components.classes[kFilePickerContractID].createInstance(kFilePickerIID);
    
      const kTitle = BookmarksUtils.getLocaleString("SelectImport");
      kFilePicker.init(window, kTitle, kFilePickerIID["modeOpen"]);
      kFilePicker.appendFilters(kFilePickerIID.filterHTML | kFilePickerIID.filterAll);
      var fileName;
      if (kFilePicker.show() != kFilePickerIID.returnCancel) {
        fileName = kFilePicker.file.path;
        if (!fileName) return;
      }
      else return;
    }
    catch (e) {
      return;
    }




    //var dsource=new RDFDataSource(fileName);
    /*var res = rdfService.GetDataSource(fileName);
    var AllBookmarksResources = res.GetAllResources();
    while (AllBookmarksResources.hasMoreElements()) {
	var urlRes = AllBookmarksResources.getNext().QueryInterface(kRDFRSCIID);

	var aParent = BMSVC.getParent(urlRes);
	if (aParent) var ptype = BookmarksUtils.resolveType(aParent);
	alert("aParent: "+aParent+" Type: "+ptype);

	//var target = BMDS.GetTarget(urlRes, urlArc, true);
    }*/
    //var res = rdfService.GetDataSource(fileName);
    //alert(res.Value);
    //var dsource=new RDF.GetDataSource("rdf:bookmarks");
    //dsource.CopyAllToDataSource(res);

    //var remoteDS = dsource.QueryInterface(Components.interfaces.nsIRDFRemoteDataSource);
    //remoteDS.Flush();
		   
    /*
var sources=recentlist.database.GetDataSources();
  var rootnode=RDF.GetResource("http://www.xulplanet.com/rdf/recent/all");
  
  while (sources.hasMoreElements()){
    try {
      dsource=sources.getNext();
      dsource=dsource.QueryInterface(Components.interfaces.nsIRDFDataSource);
  
      RDFC.Init(dsource,rootnode);
    } catch (e) {
      RDFCUtils.MakeSeq(dsource,rootnode);
      RDFC.Init(dsource,rootnode);
    }
*/

var RDFCUtils  = Components.classes["@mozilla.org/rdf/container-utils;1"].
                          createInstance(Components.interfaces.nsIRDFContainerUtils);
    
    //var rTarget = RDF.GetResource("NC:BookmarksRoot");
    try{
	RDFC.Init(inmemds, rTarget);
    } catch (e) {
      RDFCUtils.MakeSeq(inmemds,rTarget);
	RDFC.Init(inmemds, rTarget);
    }
    //inmemds.Assert(rTarget, RDF.GetResource("http://home.netscape.com/NC-rdf#Name"), RDF.GetLiteral("Debut"), true);
/*
//------------------------------------------------------------------------------
     var AllResources = BMDS.GetAllResources();
     var urlCount =0;
    while (AllResources.hasMoreElements()) {
	//var urlRes = AllResources.getNext().QueryInterface(kRDFRSCIID);
	//RDFC.AppendElement(urlRes);
	var Res = AllResources.getNext();
	var props=BMDS.ArcLabelsOut(Res);
	while(props.hasMoreElements()){
	  var prop=props.getNext();
	  prop=prop.QueryInterface(Components.interfaces.nsIRDFResource);
	  var target=BMDS.GetTarget(Res,prop,true);
	  if (target!=null) {
	  	//inmemds.Assert(Res,prop,target,true);
		RDFC.AppendElement(Res)
	  }
	}
    urlCount++;
    if (urlCount >5) break;
*/
/*
    var sourcelist=dsource1.GetAllResources();
  while(sourcelist.hasMoreElements()){
    var source=sourcelist.getNext();
    var props=dsource1.ArcLabelsOut(source);
    while(props.hasMoreElements()){
      var prop=props.getNext();
      prop=prop.QueryInterface(Components.interfaces.nsIRDFResource);
      var target=dsource1.GetTarget(source,prop,true);
      if (target!=null) dsource2.Assert(source,prop,target,true);
    }
  }
    }
*/
//------------------------------------------------------------------------------
/*

    var ALLelements = RDFC.GetElements();
var OriginalID = new Array();
     while (ALLelements.hasMoreElements()) {
	var urlRes = ALLelements.getNext().QueryInterface(kRDFRSCIID);
    OriginalID.push(urlRes); 
    }

    //alert("Count: "+elementCount);
*/
    RDFC.Init(BMDS, rTarget);
    var countBefore = parseInt(BookmarksUtils.getProperty(rTarget, gRDF_NS+"nextVal"));
    var args = [{ property: gNC_NS+"URL", literal: fileName}];
    BookmarksCommand.doBookmarksCommand(rTarget, gNC_NS_CMD+"import", args);
    var countAfter = parseInt(BookmarksUtils.getProperty(rTarget, gRDF_NS+"nextVal"));

//alert("CountBefore= "+countBefore+"CountAfter"+countAfter+"Numbers total: "+numbers);
/*
	var aParent = BMSVC.getParent(urlRes);
	if (aParent) var ptype = BookmarksUtils.resolveType(aParent);
	var urlArc = RDF.GetResource(gNCNS+"URL");
	var target = BMDS.GetTarget(urlRes, urlArc, true);
	var urlArcName = RDF.GetResource(gNCNS+"Name");
	var targetName = BMDS.GetTarget(urlRes, urlArcName, true);
	if (targetName){
	//var url = target.QueryInterface(kRDFLITIID).Value;
	if (BMSVC.isBookmarkedResource(urlRes)) alert("Resource Bookmarked: "+urlRes.Value+"\nType: "+targetName.QueryInterface(kRDFLITIID).Value);
	    //alert("UrlRes: "+urlRes.Value+" Type: "+targetName.QueryInterface(kRDFLITIID).Value);
	}

	//var target = BMDS.GetTarget(urlRes, urlArc, true);
*/

///////////////////////////////////

    var transaction = new BookmarkImportTransaction("import");
    for (var index = countBefore; index < countAfter; index++) {
      var nChildArc = RDFCU.IndexToOrdinalResource(index);
      var rChild    = BMDS.GetTarget(rTarget, nChildArc, true);
      transaction.item   .push(rChild);
      transaction.parent .push(rTarget);
      transaction.index  .push(index);
      var props=BMDS.ArcLabelsOut(rChild);
	while(props.hasMoreElements()){
	  var prop=props.getNext();
	  prop=prop.QueryInterface(Components.interfaces.nsIRDFResource);
	  var target=BMDS.GetTarget(rChild,prop,true);
	  if (target!=null) {
	  	inmemds.Assert(rChild,prop,target,true);
	  }
	}
    }
    /*
    for (var index = 1; index < countBefore; index++) {
      var nChildArc = RDFCU.IndexToOrdinalResource(index);
      var rChild    = BMDS.GetTarget(rTarget, nChildArc, true);
      var props=BMDS.ArcLabelsOut(rChild);
	while(props.hasMoreElements()){
	  var prop=props.getNext();
	  prop=prop.QueryInterface(Components.interfaces.nsIRDFResource);
	  var target=BMDS.GetTarget(rChild,prop,true);
	  if (target!=null) {
	  	inmemds.Assert(rChild,prop,target,true);
		//RDFC.AppendElement(Res)
	  }
	}
    }*/

var AllBookmarksResources = inmemds.GetAllResources();
var numbers=0;
while (AllBookmarksResources.hasMoreElements()) {
	var urlRes = AllBookmarksResources.getNext().QueryInterface(kRDFRSCIID);

	var aParent = BMSVC.getParent(urlRes);
	if (aParent) var ptype = BookmarksUtils.resolveType(aParent);
	var urlArcName = RDF.GetResource(gNCNS+"Name");
	var targetName = BMDS.GetTarget(urlRes, urlArcName, true);
	if (targetName){
	    //alert("Resource: "+urlRes.Value+"\nType: "+targetName.QueryInterface(kRDFLITIID).Value);
	}
numbers++;
	//var target = BMDS.GetTarget(urlRes, urlArc, true);
}
alert("number of elements in inmemds: "+numbers);
open("chrome://duplicates/content/ImportBookmarks.xul", "BookmarkDD-Import", "chrome,alwaysRaised,resizable,centerscreen", inmemds);


/*
    var OriginalIDNEW = new Array();
    var ALLelementsAfter = RDFC.GetElements();
    while (ALLelementsAfter.hasMoreElements()) {
	var urlRes = ALLelementsAfter.getNext().QueryInterface(kRDFRSCIID);
    OriginalIDNEW.push(urlRes); 
    }
function isAlreaydyIn(element, index, array) {
  var resul_index = OriginalID.indexOf(element);
    if(resul_index>=0) return false;
    return true;
}
var newID = OriginalIDNEW.filter(isAlreaydyIn);
//filtered = [12, 5, 8, 130, 44].filter(isBigEnough);
alert("Length="+OriginalID.length+" NEWLength="+OriginalIDNEW.length+"  NEW ID: "+newID.length);
*/


    return;
    BMSVC.transactionManager.doTransaction(transaction);
    BookmarksUtils.flushDataSource();

  }
/*****************************************************************************
    "END OF IMPORT TEST"
*****************************************************************************/

}//End of Bookmarkdd variable



// Replace function addBookmarkForTabBrowser in browser.js
// Add support for isBookmarked()
function addBookmarkForTabBrowser(aTabBrowser, aBookmarkAllTabs, aSelect)
{
    var DuplicatesBundle = document.getElementById('duplicates-strings');
    var tabsInfo = [];
    var currentTabInfo = { name: "", url: "", charset: null };
    if (Bookmarkdd.version == "1.0" && aBookmarkAllTabs) {
	aSelect = aBookmarkAllTabs;
	if (aSelect) aBookmarkAllTabs = true;
    }

    const activeBrowser = aTabBrowser.selectedBrowser;
    const browsers = aTabBrowser.browsers;
    for (var i = 0, j = 0; i < browsers.length; ++i) {
	var webNav = browsers[i].webNavigation;
	var url = webNav.currentURI.spec;
	var name = "";
	var charSet, description;
	try {
	    var doc = webNav.document;
	    name = doc.title || url;
	    charSet = doc.characterSet;
	    if (Bookmarkdd.version == "1.5") description = BookmarksUtils.getDescriptionFromDocument(doc);
	} catch (e) {
	    name = url;
	}

	var isDuplicate = Bookmarkdd.isBookmarked(url);
	if(isDuplicate) {
	    var messageURL = DuplicatesBundle.getString("followingURL")+"\n";
	    messageURL += url+"\n";
	    messageURL += DuplicatesBundle.getString("inFolder")+"\n";
	    messageURL += isDuplicate.parents.join("\n")+"\n";
	    messageURL += DuplicatesBundle.getString("alreadyIn-Confirm");

	    var message = DuplicatesBundle.getString("alreadyIn-Folder")+"\n";
	    message += isDuplicate.parents.join("\n")+"\n";
	    message += DuplicatesBundle.getString("alreadyIn-Confirm");

	    if(aBookmarkAllTabs) {
		if (! BddConfirm(messageURL, isDuplicate.RDFnodes)) continue;
	    } else if (browsers[i] == activeBrowser) {
		if (! BddConfirm(message, isDuplicate.RDFnodes)) return false;
	    }
	}

	tabsInfo[j] = { name: name, url: url, charset: charSet, description: description };
	if (browsers[i] == activeBrowser) currentTabInfo = tabsInfo[j];
	j++;
    }

    if (tabsInfo.length == 0) {
	alert (DuplicatesBundle.getString("NoNewURL"));
	return false;
    }

    var dialogArgs = currentTabInfo;
    if (Bookmarkdd.version == "1.5" && aBookmarkAllTabs) {
	dialogArgs = { name: gNavigatorBundle.getString("bookmarkAllTabsDefault") };
	dialogArgs.bBookmarkAllTabs = true;
    }


    dialogArgs.objGroup = tabsInfo;
    if(Bookmarkdd.version == "1.5") {
	return window.openDialog("chrome://browser/content/bookmarks/addBookmark2.xul", "", ADD_BM_DIALOG_FEATURES, dialogArgs);
    } else if (Bookmarkdd.version == "1.0") {
	return window.openDialog("chrome://browser/content/bookmarks/addBookmark2.xul", "", "centerscreen,chrome,dialog,resizable,dependent", currentTabInfo.name, currentTabInfo.url, null, currentTabInfo.charset, "addGroup" + (aSelect ? ",group" : ""), tabsInfo);


    }
}

//BookmarkDD uses its own confirm window to replace default icon without overlay icon for all Browser's confirm
function BddConfirm(aText, aRDFnodes){

    window.openDialog("chrome://duplicates/content/ConfirmOverlay.xul", "BookmarkDD-Confirm", "chrome,modal,alwaysRaised,resizable,centerscreen", aText, aRDFnodes);

    return Bookmarkdd.userResponse;
}

//Verify which Firefox version is used
try {
    if (gNC_NS) Bookmarkdd.version="1.5";
}
catch(e){
    Bookmarkdd.version = "1.0";
}


//Add properties to URLFound Array
function URLConstructor(){
    this.parents = new Array();
    this.RDFnodes = new Array();
}


// Replace BookmarksUtils method by our functions to detect duplicate URL
BookmarksUtils.insertAndCheckSelection = Bookmarkdd.insertAndCheckURL; 
BookmarksUtils.addBookmark = Bookmarkdd.addBookmarkURL; 
BookmarksUtils.addLivemark = Bookmarkdd.addLivemark; 
//BookmarksCommand.importBookmarksFromFile = Bookmarkdd.importBookmarksFromFile; 

